--[[
	---------------------------------------------------------
    calca-Speed for my Lua Programms
    
    Requires transmitter firmware 4.22 or higher.
    
    Lua application for JETI DC/DS transmitters

	Copyright (c) 2023, Walter Loetscher
	All rights reserved.

	License: Share alike
	Can be used and changed non commercial
--]]
collectgarbage()
--------------------------------------------------------------------------------
-- Locals for application
local mainSwitch, fTO, flapTO, fLnd, flapLnd, sensSpeed, speedSe, speedSeId, speedSePa
local kmhF, msF, ktsF, mphF = 0.36, 1, 1.94384, 2.23694
local kmhI, msI, ktsI, mphI = 3.6
local v1, vR, vMinClean, vMinTO, vMinFF
local v1met, vRmet, vCmet, vTOmet, vFFmet = 0,0,0,0,0
local sensorLalist = { "..." }
local sensorIdlist = { "..." }
local sensorPalist = { "..." }
local dispMode, sourceMode, modeChange, lastMode = 1, 1, false, 0
local flapState, flightState, mps, simSwitch, simSpeed, sim, oldSim, speedSim, rawSpeed, metSpeed, dispSpeed, maxSpeed = 0,0,0,0,0,0,0,0,0,0,0,0
local fDelay, gDelay, units, msw, oldmsw, message = 0,0,"def",0,0," "
local alarmT, lastAlarm, v1Alarm, rotAlarm, aChange, triggerT, trigger, triggerDisp = 0,0,false,false, false, 0, 0,0,0
local timeElapsed, alTime, spread, mSpread, weightFactor, mWeight, weightCorr = true, 0, 2, 2, 8, 0, 1
local dix12, dUnit
--------------------------------------------------------------------------------
-- Read and set translations
local function setLanguage()
    local lng=system.getLocale()
    local file = io.readall("Apps/calca-Spd/calca-Spd.jsn")
    local obj = json.decode(file)
    if(obj) then
        dix12 = obj[lng] or obj[obj.default]
    end
end
--------------------------------------------------------------------------------
local function debugT()
	lcd.drawText(0,0,string.format("Flight: %d",flightState),FONT_NORMAL)
	lcd.drawText(0,15,string.format("Flap: %d",flapState),FONT_NORMAL)
	if Calca_dispGas then 
	lcd.drawText(80,15,string.format("G: %.1f".."%%",Calca_dispGas),FONT_NORMAL)
	end	
	if dispMode == 1 then
		lcd.drawText(10,32,string.format("Speed: %d".." "..units,dispSpeed),FONT_BOLD) else
		lcd.drawText(10,32,string.format("Speed: %.1f".." "..units,dispSpeed),FONT_BOLD)
	end
	lcd.drawText(0,50,string.format("%.1f".."ms",metSpeed/10),FONT_NORMAL)
	lcd.drawText(60,50,triggerDisp,FONT_NORMAL)
	lcd.drawText(50,0,string.format("%18s",message))
end
--------------------------------------------------------------------------------
local function printTelem()
	if flightState == 1 then lcd.drawText(0,0,dix12.takeOff,FONT_MINI) else lcd.drawText(0,0,dix12.flight,FONT_MINI) end
	lcd.drawText(95,0, message,FONT_MINI)
	lcd.drawText(55,0,string.format("Flap: %d",flapState),FONT_MINI)
	if Calca_dispGas then 
	lcd.drawText(5,55,string.format("cap: %.1f".."%%",Calca_dispGas),FONT_MINI)
	end	
	if dispMode == 1 then
		lcd.drawText(10,15,string.format("Speed: %d".." "..units,dispSpeed),FONT_BOLD)
		lcd.drawText(70,55,string.format("max %d".." "..units,maxSpeed),FONT_MINI) else
		lcd.drawText(10,15,string.format("Speed: %.1f".." "..units,dispSpeed),FONT_BOLD)
		lcd.drawText(70,55,string.format("max %.1f".." "..units,maxSpeed),FONT_MINI)
	end
	lcd.drawText(25,32,"("..triggerDisp..")",FONT_NORMAL)	
end
--------------------------------------------------------------------------------
-- Read available sensors for user to select
local function readSensors()
    local sensors = system.getSensors()
    local format = string.format
    local insert = table.insert
    for i, sensSpeed in ipairs(sensors) do
        if (sensSpeed.label ~= "") then
            insert(sensorLalist, format("%s", sensSpeed.label))
            insert(sensorIdlist, format("%s", sensSpeed.id))
            insert(sensorPalist, format("%s", sensSpeed.param))
        end
    end
end
----------------------------------------------------------------------
local function convert()
	if dispMode == 1 then
	triggerDisp = string.format("min: %d", trigger * kmhF) .. " kmh"
	elseif dispMode == 2 then
	triggerDisp = string.format("min: %.1f", trigger / 10) .. " ms"
	elseif dispMode == 3 then
	triggerDisp = string.format("min: %.1f", trigger * ktsF / 10) .. " kts"
	elseif dispMode == 4 then
	triggerDisp = string.format("min: %.1f", trigger * mphF / 10) .. " mph"
	end
end
--------------------------------------------------------------------------------
local function convUnits()
	if dispMode == 1 then
		v1met = v1 / kmhF 
		vRmet = vR / kmhF 
		vCmet = vMinClean / kmhF	
		vTOmet = vMinTO / kmhF 
		vFFmet = vMinFF / kmhF
		mSpread = spread / kmhF
		elseif dispMode == 2 then
		v1met = v1 / msF 
		vRmet = vR / msF 
		vCmet = vMinClean / msF 
		vTOmet = vMinTO / msF 
		vFFmet = vMinFF / msF
		mSpread = spread / msF
		elseif dispMode == 3 then
		v1met = v1 / ktsF 
		vRmet = vR / ktsF 
		vCmet = vMinClean / ktsF 
		vTOmet = vMinTO / ktsF 
		vFFmet = vMinFF / ktsF
		mSpread = spread / ktsF
		elseif dispMode == 4 then
		v1met = v1 / mphF 
		vRmet = vR / mphF 
		vCmet = vMinClean / mphF 
		vTOmet = vMinTO / mphF 
		vFFmet = vMinFF / mphF
		mSpread = spread / mphF
	end
end
--------------------------------------------------------------------------------
-- Actions when settings changed
local function flapTOChanged(value)
    local pSave = system.pSave
	flapTO = value
	pSave("flapTO", value)
end

local function flapLndChanged(value)
    local pSave = system.pSave
	flapLnd = value
	pSave("flapLnd", value)
end

local function auChanged(value)
    local pSave = system.pSave
	mainSwitch = value
	pSave("mainSwitch", value)
	aChange = true
end

local function sensorChanged(value)
    local pSave = system.pSave
    local format = string.format
    speedSe = value
    speedSeId = format("%s", sensorIdlist[speedSe])
    speedSePa = format("%s", sensorPalist[speedSe])
    if (speedSeId == "...") then
        speedSeId = 0
        speedSePa = 0 
    end
    pSave("speedSe", value)
    pSave("speedSeId", speedSeId)
    pSave("speedSePa", speedSePa)
end

local function dmChanged(value)
	lastMode =  dispMode
	dispMode=value
	system.pSave("dispMode",value) 
	modeChange = true
end

local function smChanged(value)
	sourceMode=value
	system.pSave("sourceMode",value)
	modeChange = true
end

local function siChanged(value)														-- Simulation 1=on
	simSwitch=value
	system.pSave("simSwitch",value)
end
local function spChanged(value)														-- Simulation 1=on
	simSpeed=value
	system.pSave("simSpeed",value)
end

local function m1Changed(value)
	v1=value
	system.pSave("v1",value)
	convUnits()
end
local function mrChanged(value)
	vR=value
	system.pSave("vR",value)
	convUnits()
end
local function mcChanged(value)
	vMinClean=value
	system.pSave("vMinClean",value)
	convUnits()
end
local function mtChanged(value)
	vMinTO=value
	system.pSave("vMinTO",value)
	convUnits()
end
local function mfChanged(value)
	vMinFF=value
	system.pSave("vMinFF",value)
	convUnits()
end
local function srChanged(value)
	spread=value
	system.pSave("spread",value)
	convUnits()
end
local function wfChanged(value)
	weightFactor=value
	system.pSave("weightFactor",value)
	convUnits()
end
--------------------------------------------------------------------------------
local function newMode()	
	if lastMode == 1 then
	v1met = v1 / kmhF 
	vRmet = vR / kmhF 
	vCmet = vMinClean / kmhF	
	vTOmet = vMinTO / kmhF 
	vFFmet = vMinFF / kmhF
	mSpread = spread / kmhF
	elseif lastMode == 2 then
	v1met = v1 / msF 
	vRmet = vR / msF 
	vCmet = vMinClean / msF 
	vTOmet = vMinTO / msF 
	vFFmet = vMinFF / msF
	mSpread = spread / msF
	elseif lastMode == 3 then
	v1met = v1 / ktsF 
	vRmet = vR / ktsF 
	vCmet = vMinClean / ktsF 
	vTOmet = vMinTO / ktsF 
	vFFmet = vMinFF / ktsF
	mSpread = spread / ktsF
	elseif lastMode == 4 then
	v1met = v1 / mphF 
	vRmet = vR / mphF 
	vCmet = vMinClean / mphF 
	vTOmet = vMinTO / mphF 
	vFFmet = vMinFF / mphF
	mSpread = spread / mphF
	end	
	if dispMode == 1 then
	units = "kmh"
	v1 = v1met * kmhF
	vR = vRmet * kmhF 
	vMinClean = vCmet * kmhF 
	vMinTO = vTOmet * kmhF 
	vMinFF = vFFmet * kmhF
	spread = mSpread * kmhF
	elseif dispMode == 2 then
	units = "ms"
	v1 = v1met 
	vR = vRmet 
	vMinClean = vCmet 
	vMinTO = vTOmet 
	vMinFF = vFFmet
	spread = mSpread
	elseif dispMode == 3 then
	units = "kts"
	v1 = v1met * ktsF 
	vR = vRmet * ktsF 
	vMinClean = vCmet * ktsF 
	vMinTO = vTOmet * ktsF 
	vMinFF = vFFmet * ktsF
	spread = mSpread * ktsF
	elseif dispMode == 4 then
	units = "mph"
	v1 = v1met * mphF 
	vR = vRmet * mphF 
	vMinClean = vCmet * mphF 
	vMinTO = vTOmet * mphF 
	vMinFF = vFFmet * mphF
	spread = mSpread * mphF
	end		
	m1Changed(math.floor(v1+0.5))
	mrChanged(math.floor(vR+0.5))
	mcChanged(math.floor(vMinClean+0.5))
	mtChanged(math.floor(vMinTO+0.5))
	mfChanged(math.floor(vMinFF+0.5))
	srChanged(math.floor(spread+0.5))
	lastMode = dispMode
	form.reinit()
	modeChange = false
end
--------------------------------------------------------------------------------
local function initMode()
	lastMode = dispMode
	flightState = 1
	newMode()
end
--------------------------------------------------------------------------------
local function playAlarm(value)
	
	if value == 1 then system.playFile("/Apps/Calca-Spd/audio_01.wav", AUDIO_BACKGROUND) message = "A1"
	elseif value == 2 then system.playFile("/Apps/Calca-Spd/audio_02.wav", AUDIO_BACKGROUND) message = "A2"
	elseif value == 3 then system.playFile("/Apps/Calca-Spd/audio_03.wav", AUDIO_BACKGROUND) message = "A3"
	elseif value == 4 then system.playFile("/Apps/Calca-Spd/audio_04.wav", AUDIO_BACKGROUND) message = "A4"
	elseif value == 10 then system.playFile("/Apps/Calca-Spd/audio_05.wav", AUDIO_BACKGROUND) message = "slow!"
	elseif value == 20 then system.playFile("/Apps/Calca-Spd/audio_06.wav", AUDIO_BACKGROUND) message = "very slow!"
	end
end
--------------------------------------------------------------------------------
local function alarme()
	if msw == 1 then return end
	if flightState == 1 then	
		alarmT = 0	
		if ((v1met > 0) and (metSpeed >= v1met)) and (v1Alarm == false) then
			system.playFile("/Apps/Calca-Spd/v1.wav", AUDIO_IMMEDIATE)
			message = "v1"
--			print("v1")
			v1Alarm = true
		end
		if ((vRmet > 0) and (metSpeed >= vRmet)) and (rotAlarm == false) then
			system.playFile("/Apps/Calca-Spd/rotate.wav", AUDIO_IMMEDIATE)
			message = "rotate"
--			print("rotate")
			rotAlarm = true
		end
	end
	if flightState == 2 then	
		if v1Alarm == true then v1Alarm = false end
		if rotAlarm == true then rotAlarm = false end
		if metSpeed > trigger and metSpeed < trigger +mSpread+1 then alarmT = 4
		elseif metSpeed > trigger +mSpread and metSpeed < trigger +(2*mSpread)+1 then alarmT = 3
		elseif metSpeed > trigger +(2*mSpread) and metSpeed < trigger +(3*mSpread)+1 then alarmT = 2
		elseif metSpeed > trigger +(3*mSpread) and metSpeed < trigger +(4*mSpread)+1 then alarmT = 1
		elseif metSpeed > trigger -(2*mSpread) and metSpeed < trigger +1 then alarmT = 10
		elseif metSpeed > trigger -(4*mSpread) and metSpeed < trigger -(2*mSpread)+1 then alarmT = 20
		else alarmT = 0 message = "ready" return
		end		
		if alarmT ~= lastAlarm then
		playAlarm(alarmT)
		lastAlarm = alarmT
		alTime = system.getTime()
		elseif system.getTime() > alTime then
		alTime = system.getTime()
		playAlarm(alarmT)
		end	
	end
end
--------------------------------------------------------------------------------
local function checkState()
	if (fTO == 0) and (fLnd == 0) then flapState = 0
	elseif (fTO == 1) and (fLnd == 1) then flapState = 2
	elseif fTO == 1 then flapState = 1
	elseif fLnd == 1 then flapState = 2
	else flapState = 0
	end
	if flightState == 1 then
		if metSpeed > 120 then fDelay = fDelay + 1 else fDelay = 0 end
		if fDelay == 200 then flightState = 2 end
	end
	if flightState == 2 then
		if metSpeed < 100 then gDelay = gDelay + 1 else gDelay = 0 end
		if gDelay == 200 then flightState = 1 end
	end	
end
--------------------------------------------------------------------------------
local function checkSwitches()
	fTO  = system.getInputsVal(flapTO)
	if fTO == nil then fTO = 0 end
	fLnd  = system.getInputsVal(flapLnd)
	if fLnd == nil then fLnd = 0 end	
	sim = (system.getInputsVal(simSwitch)) or 0
	if sim ~= oldSim then oldSim = sim v1Alarm = false rotAlarm = false message = " " end
	speedSim = (system.getInputsVal(simSpeed)) or 0	
	msw = (system.getInputsVal(mainSwitch)) or 0
	if msw ~= oldmsw then aChange = true oldmsw = msw end 
    sensSpeed = system.getSensorByID(speedSeId, speedSePa)
    if(sensSpeed and sensSpeed.valid) then
        rawSpeed = tonumber(string.format("%.1f", sensSpeed.value))
    end
    collectgarbage()
end
--------------------------------------------------------------------------------
local function getSpeed()
	if sim == 1 then rawSpeed = speedSim * 100 end
	if sourceMode == 1 then metSpeed = math.floor((rawSpeed/kmhF)+0.5) elseif
	sourceMode == 2 then metSpeed = math.floor((rawSpeed*10)+0.5) elseif
	sourceMode == 3 then metSpeed = math.floor((rawSpeed/ktsF*10)+0.5) elseif
	sourceMode == 4 then metSpeed = math.floor((rawSpeed/mphF*10)+0.5)
	end
	if dispMode == 1 then dispSpeed = math.floor((metSpeed * kmhF)+0.5) elseif
	dispMode == 2 then dispSpeed = metSpeed / 10 elseif
	dispMode == 3 then dispSpeed = metSpeed * ktsF / 10 elseif
	dispMode == 4 then dispSpeed = metSpeed * mphF / 10
	end
	if dispSpeed >= maxSpeed then maxSpeed = dispSpeed end
end
--------------------------------------------------------------------------------
local function weightComp()
	weightCorr = 1 + ((Calca_dispGas - 20) * weightFactor / 10000)
end
--------------------------------------------------------------------------------
local function loop()
	if Calca_dispGas then weightComp() end
	if lastMode == 0 then initMode() end
	if modeChange == true then newMode() end
	if aChange == true then
		if msw == 1.0 then message = "Alarm Off" else message = "Alarm On"
		aChange = false
		end	
	end	
	if flapState == 2 then triggerT = vFFmet * weightCorr
		elseif flapState == 1 then triggerT = vTOmet * weightCorr
		else triggerT = vCmet * weightCorr
	end
	trigger = math.floor(triggerT + 0.5)
	convert()	
	checkState()
	checkSwitches()
	getSpeed()
	alarme()
    collectgarbage()
end
--------------------------------------------------------------------------------
-- Draw the main form (Application inteface)
local function initForm()
    local fw = tonumber(string.format("%.2f", system.getVersion()))
    if(fw >= 4.22)then
		local versTable = {dix12.kmh,dix12.ms,dix12.kts, dix12.mph}		
        local form, addRow, addLabel = form, form.addRow ,form.addLabel
        local addIntbox, addCheckbox = form.addIntbox, form.addCheckbox
        local addSelectbox, addInputbox = form.addSelectbox, form.addInputbox		
		addLabel({label= "                        "..dix12.sensors, font=FONT_NORMAL})       
        addRow(2)
        addLabel({label=dix12.spdTel, width=190, font=FONT_BOLD})
        addSelectbox(sensorLalist, speedSe, true, sensorChanged, {font=FONT_BOLD})       		
		addRow(2)
		addLabel({label=dix12.flapTO, width=190, font=FONT_BOLD})
		addInputbox(flapTO, true, flapTOChanged, {font=FONT_BOLD})
		addRow(2)
		addLabel({label=dix12.flapLnd, width=190, font=FONT_BOLD})
		addInputbox(flapLnd, true, flapLndChanged, {font=FONT_BOLD})		
		addRow(2)
		addLabel({label= dix12.modeD, width=190, font=FONT_BOLD})
		addSelectbox(versTable, dispMode, true, dmChanged, {font=FONT_BOLD})
		addRow(2)
		addLabel({label= dix12.modeS, width=190, font=FONT_BOLD})
		addSelectbox(versTable, sourceMode, true, smChanged, {font=FONT_BOLD})		
		addRow(2)
		addLabel({label=dix12.audibles, width=190, font=FONT_BOLD})
		addInputbox(mainSwitch, true, auChanged)
		addRow(2)
		addLabel({label= "Simulation", width=190, font=FONT_BOLD})
		addInputbox(simSwitch,true,siChanged, {font=FONT_BOLD})
		addRow(2)
		addLabel({label= "Speed", width=190, font=FONT_BOLD})
		addInputbox(simSpeed,true,spChanged, {font=FONT_BOLD})
		addRow(1)
		addLabel({label= dix12.prop, width=190, font=FONT_NORMAL})
		if dispMode == 1 then
			addLabel({label= "----------------------------------------------------------"})
			addLabel({label= "                        "..dix12.speeds.."kmh", font=FONT_NORMAL})		
			addRow(2)
			addLabel({label= dix12.v1, width=190, font=FONT_BOLD})
			addIntbox(v1,0,500,0,0,1,m1Changed, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vR, width=190, font=FONT_BOLD})
			addIntbox(vR,0,500,0,0,1,mrChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinClean, width=190, font=FONT_BOLD})
			addIntbox(vMinClean,0,500,0,0,1,mcChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinTO, width=190, font=FONT_BOLD})
			addIntbox(vMinTO,0,500,0,0,1,mtChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinFF, width=190, font=FONT_BOLD})
			addIntbox(vMinFF,0,500,0,0,1,mfChanged, {font=FONT_BOLD})			
			addRow(2)
			addLabel({label= dix12.spread, width=190, font=FONT_BOLD})
			addIntbox(spread,2,10,5,0,1,srChanged, {font=FONT_BOLD})
		else
		
			if dispMode == 2 then dUnit = "ms"
			elseif dispMode == 3 then dUnit = "kts"
			else dUnit = "mph"
			end
		
			addLabel({label= "----------------------------------------------------------"})
			addLabel({label= "                        "..dix12.speeds..dUnit, font=FONT_NORMAL})		
			addRow(2)
			addLabel({label= dix12.v1, width=190, font=FONT_BOLD})
			addIntbox(v1,0,5000,0,1,1,m1Changed, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vR, width=190, font=FONT_BOLD})
			addIntbox(vR,0,5000,0,1,1,mrChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinClean, width=190, font=FONT_BOLD})
			addIntbox(vMinClean,0,5000,0,1,1,mcChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinTO, width=190, font=FONT_BOLD})
			addIntbox(vMinTO,0,5000,0,1,1,mtChanged, {font=FONT_BOLD})
			addRow(2)
			addLabel({label= dix12.vMinFF, width=190, font=FONT_BOLD})
			addIntbox(vMinFF,0,5000,0,1,1,mfChanged, {font=FONT_BOLD})			
			addRow(2)
			addLabel({label= dix12.spread, width=190, font=FONT_BOLD})
			addIntbox(spread,2,20,14,1,1,srChanged, {font=FONT_BOLD})
		end
		addLabel({label= "----------------------------------------------------------"})
		addLabel({label= dix12.remark, width=190, font=FONT_NORMAL})
        addRow(2)
		addLabel({label= dix12.weightFactor, width=220, font=FONT_BOLD})
		addIntbox(weightFactor,0,15,8,0,1,wfChanged, {font=FONT_BOLD})
        addRow(1)
        addLabel({label="Powered by Swiss-Aerodesign.com "..dix12.version.." ", font=FONT_MINI, alignRight=true})
    else
        local addRow, addLabel = form.addRow ,form.addLabel
        addRow(1)
        addLabel({label="Please update, min. fw 4.22 required!"})
    end
end
--------------------------------------------------------------------------------
local function init()
    local pLoad = system.pLoad	
	flapTO = pLoad("flapTO")
	flapLnd = pLoad("flapLnd")
    speedSe = pLoad("speedSe", 0)
    speedSeId = pLoad("speedSeId", 0)
    speedSePa = pLoad("speedSePa", 0)
	dispMode = system.pLoad("dispMode",1)
	sourceMode = system.pLoad("sourceMode",1)
	mainSwitch = system.pLoad("mainSwitch")	
	simSwitch = system.pLoad("simSwitch",nil)												-- Simulation 1 = on
	simSpeed = system.pLoad("simSpeed",nil)													-- Anpassung Speed	
	v1 = system.pLoad("v1",0)			                                		
	vR = system.pLoad("vR",0)			                                		
	vMinClean = system.pLoad("vMinClean",0)			                                		
	vMinTO = system.pLoad("vMinTO",0)			                                		
	vMinFF = system.pLoad("vMinFF",0)
	spread = system.pLoad("spread",2)
	weightFactor = system.pLoad("weightFactor",8)	
    system.registerForm(1, MENU_MAIN, dix12.appName, initForm)
--	system.registerTelemetry(1,dix12.appName.." "..dix12.version,2,debugT)
	system.registerTelemetry(1,dix12.appName.." "..dix12.version,2,printTelem)
    readSensors()
    collectgarbage()
end
--------------------------------------------------------------------------------
setLanguage()
collectgarbage()
return {init=init, loop=loop, author="Walter Loetscher", version= dix12.version, name= dix12.appName}
